function res = perceptiveFMeasure(Mref,Mest,onsetThreshold,coef)
% Perceptive F-measure, as defined in [1].
%
% Usage:
%     res =
%     perceptiveFMeasure(Mref,transcription,onsetThreshold,coef)
%
% Inputs:
%   - Mref: reference matrix
%   - Mest: transcription matrix
%   - onsetThreshold (optionnal): threshold, in seconds, for onset
%   accuracy. Default: .150.
%   - coef (optionnal): vector of perceptive coefficients. Default: loaded
%   from file subjectiveCoef.mat.
%
% Outputs:
%   - res.percFmeas: perceptive F-measure.
%   - res.Fmeas: F-measure.
%   - res.NTP: number of true positives.
%   - res.NFP8: number of octave-related false positives.
%   - res.NFP5: number of fifth-related false positives.
%   - res.NFPx: number of other false positives.
%   - res.NFN: number of true negatives
%   - res.aveOffsetDiff: average offset error
%   - res.aveOnsetDiff: average onset error
%
%   Matrix are composed of 1 note per row and three columns: pitch (MIDI
%   scale), onset time (seconds), offset time (seconds)
%
%
% [1] Perceptually-based evaluation of the errors usually made when
% automatically transcribing music, Adrien DANIEL, Valentin EMIYA, Bertrand
% DAVID, Proc. of ISMIR 2008.
%
% Copyright 2008 by V. Emiya, TELECOM ParisTech, Paris, France.

if nargin<3
    onsetThreshold = .150;
end
if nargin<4
    load perceptiveFMeasure.mat
end

%% FP, FN and TP search
IFN = zeros(0,1); % indexes of False Negatives in Mref
ITP = zeros(0,2); % indexes of True Positives in (Mref,Mest)
for k=1:size(Mref,1)
    I = find(Mest(:,1)==Mref(k,1) & Mest(:,3)>Mref(k,2) & Mest(:,2)<Mref(k,3) & abs(Mref(k,2)-Mest(:,2))<onsetThreshold);
    if isempty(I)
        IFN = [IFN;k];
    else
        [dum J] = sort(Mest(I,2));
        ITP = [ITP;[k, I(J(1))]];
    end
end
IFP = find(~ismember(1:size(Mest,1),ITP(:,2)))'; % indexes of False Positives in Mest
Ncor = size(ITP,1);

% onset precision, offset precision
onsetDiff = zeros(Ncor,2); % for each TP, onset difference in s and in % of duration
offsetDiff = zeros(Ncor,2); % for each TP, offset difference in s and in % of duration
for k=1:Ncor
    dur = diff(Mref(ITP(k,1),2:3));
    onsetDiff(k,1) = Mest(ITP(k,2),2)-Mref(ITP(k,1),2);
    onsetDiff(k,2) = onsetDiff(k,1)/dur;
    offsetDiff(k,1) = Mest(ITP(k,2),3)-Mref(ITP(k,1),3);
    offsetDiff(k,2) = offsetDiff(k,1)/dur;
end

% octave, fifth, other pitch difference
IFP8 = zeros(size(IFP,1),2); % indexes of octave FP in (Mref,Mest)
IFP5 = zeros(size(IFP,1),2); % indexes of fifth FP in (Mref,Mest)
IFPx = zeros(size(IFP,1),1); % indexes of other FP in Mest
n8 = 1;
n5 = 1;
nx = 1;
for k=1:length(IFP)
    n = IFP(k);
    I = find(Mest(n,3)>Mref(:,2) & Mest(n,2)<Mref(:,3));
    I8 = find(abs(Mref(I,1)-Mest(n,1))==12,1,'first');
    I5 = find(abs(Mref(I,1)-Mest(n,1))==7,1,'first');
    if ~isempty(I8)
        IFP8(n8,:) = [I(I8),n];
        n8 = n8+1;
    elseif ~isempty(I5)
        IFP5(n5,:) = [I(I5),n];
        n5 = n5+1;
    else
        IFPx(nx) = n;
        nx = nx+1;
    end
end
IFP8 = IFP8(1:n8-1,:);
IFP5 = IFP5(1:n5-1,:);
IFPx = IFPx(1:nx-1,:);

detailedScores = [size(IFP8,1),size(IFP5,1),size(IFPx,1),length(IFN),sqrt(mean(offsetDiff(:,2).^2)).*size(ITP,1),sqrt(mean(onsetDiff(:,2).^2)).*size(ITP,1)];
res.percFmeas = size(ITP,1)./(size(ITP,1)+detailedScores*coef);
res.Fmeas = size(ITP,1)./(size(ITP,1)+.5*size(IFP,1)+.5*size(IFN,1));
res.NTP = size(ITP,1);
res.NFP8 = size(IFP8,1);
res.NFP5 = size(IFP5,1);
res.NFPx = size(IFPx,1);
res.NFN = size(IFN,1);
res.aveOffsetDiff = sqrt(mean(offsetDiff(:,2).^2));
res.aveOnsetDiff = sqrt(mean(onsetDiff(:,2).^2))

return